/*  ctype.h
 *  ANSI C Runtime Library 
 */

#ifndef _CTYPE_H
#define _CTYPE_H
#define _UPPER          0x01
#define _LOWER          0x02
#define _DIGIT          0x04
#define _SPACE          0x08
#define _PUNCT          0x10
#define _CONTROL        0x20
#define _HEXDIGIT       0x40
#define _UNDERSCORE     0x80

extern unsigned char _ctype_[];
extern int __k;                 /* temporary storage for the to* macros */

int isalnum(int c);
#define isalnum(ch) (((_ctype_+1)[(ch)])&(_UPPER|_LOWER|_DIGIT))
int isalpha(int c);
#define isalpha(ch) (((_ctype_+1)[(ch)])&(_UPPER|_LOWER))
int iscntrl(int c);
#define iscntrl(ch) (((_ctype_+1)[(ch)])&(_CONTROL))
int isdigit(int c);
#define isdigit(ch) (((_ctype_+1)[(ch)])&(_DIGIT))
int isgraph(int c);
#define isgraph(ch) (((_ctype_+1)[(ch)])&(_UPPER|_LOWER|_DIGIT|_PUNCT))
int islower(int c);
#define islower(ch) (((_ctype_+1)[(ch)])&(_LOWER))
int isprint(int c);
#define isprint(ch) (__k=(ch),!iscntrl(__k)&&__k>=0)    /* beware of EOF */
int ispunct(int c);
#define ispunct(ch) (((_ctype_+1)[(ch)])&(_PUNCT))
int isspace(int c);
#define isspace(ch) (((_ctype_+1)[(ch)])&(_SPACE))
int isupper(int c);
#define isupper(ch) (((_ctype_+1)[(ch)])&(_UPPER))
int isxdigit(int c);
#define isxdigit(ch) (((_ctype_+1)[(ch)])&(_DIGIT|_HEXDIGIT))
int tolower(int c);
#define tolower(ch) (__k=(ch),isupper(__k)?(__k-('A'-'a')):__k)
int toupper(int c);
#define toupper(ch) (__k=(ch),islower(__k)?(__k-('a'-'A')):__k)

/* old versions included against the standard */
#define isascii(ch) ((unsigned)(ch)<=0x7f)
#define toascii(ch) ((ch)&0x7f)
#define iscntl(ch)  (((_ctype_+1)[(ch)])&(_CONTROL))

/* extensions */
#define _tolower(ch) ((ch)+('a'-'A'))       /* old, simple versions */
#define _toupper(ch) ((ch)-('a'-'A'))
#define _isprint(ch) (!iscntrl(ch))
#define iscsym(ch)  (((_ctype_+1)[(ch)])&(_UPPER|_LOWER|_DIGIT|_UNDERSCORE))
#define iscsymf(ch) (((_ctype_+1)[(ch)])&(_UPPER|_LOWER|_UNDERSCORE))
#endif  /* _CTYPE_H */
